# Stubs for xlwt.CompoundDoc (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class XlsDoc:
    SECTOR_SIZE: int = ...
    MIN_LIMIT: int = ...
    SID_FREE_SECTOR: int = ...
    SID_END_OF_CHAIN: int = ...
    SID_USED_BY_SAT: int = ...
    SID_USED_BY_MSAT: int = ...
    book_stream_sect: Any = ...
    dir_stream: str = ...
    dir_stream_sect: Any = ...
    packed_SAT: str = ...
    SAT_sect: Any = ...
    packed_MSAT_1st: str = ...
    packed_MSAT_2nd: str = ...
    MSAT_sect_2nd: Any = ...
    header: str = ...
    def __init__(self) -> None: ...
    book_stream_len: Any = ...
    def save(self, file_name_or_filelike_obj: Any, stream: Any) -> None: ...



# Stubs for xlwt.ExcelFormulaParser (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .ExcelMagic import *
from . import antlr
from typing import Any

class FormulaParseException(Exception): ...

SKIP: Any
INVALID_TYPE: Any
EOF_TYPE: Any
EOF: Any
NULL_TREE_LOOKAHEAD: Any
MIN_USER_TYPE: Any
TRUE_CONST: int
FALSE_CONST: int
STR_CONST: int
NUM_CONST: int
INT_CONST: int
FUNC_IF: int
FUNC_CHOOSE: int
NAME: int
QUOTENAME: int
EQ: int
NE: int
GT: int
LT: int
GE: int
LE: int
ADD: int
SUB: int
MUL: int
DIV: int
POWER: int
PERCENT: int
LP: int
RP: int
LB: int
RB: int
COLON: int
COMMA: int
SEMICOLON: int
REF2D: int
REF2D_R1C1: int
BANG: int
CONCAT: int

class Parser(antlr.LLkParser):
    tokenNames: Any = ...
    rpn: bytes = ...
    sheet_references: Any = ...
    xcall_references: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def formula(self) -> None: ...
    def expr(self, arg_type: Any) -> None: ...
    def prec0_expr(self, arg_type: Any) -> None: ...
    def prec1_expr(self, arg_type: Any) -> None: ...
    def prec2_expr(self, arg_type: Any) -> None: ...
    def prec3_expr(self, arg_type: Any) -> None: ...
    def prec4_expr(self, arg_type: Any) -> None: ...
    def prec5_expr(self, arg_type: Any) -> None: ...
    def primary(self, arg_type: Any) -> None: ...
    def sheet(self): ...
    def expr_list(self, arg_type_list: Any, min_argc: Any, max_argc: Any): ...

def mk_tokenSet_0(): ...



# Stubs for xlwt.Row (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Row:
    height: int = ...
    has_default_height: int = ...
    height_mismatch: int = ...
    level: int = ...
    collapse: int = ...
    hidden: int = ...
    space_above: int = ...
    space_below: int = ...
    def __init__(self, rowx: Any, parent_sheet: Any) -> None: ...
    def get_height_in_pixels(self): ...
    def set_style(self, style: Any) -> None: ...
    def get_xf_index(self): ...
    def get_cells_count(self): ...
    def get_min_col(self): ...
    def get_max_col(self): ...
    def get_row_biff_data(self): ...
    def insert_cell(self, col_index: Any, cell_obj: Any) -> None: ...
    def insert_mulcells(self, colx1: Any, colx2: Any, cell_obj: Any) -> None: ...
    def get_cells_biff_data(self): ...
    def get_index(self): ...
    def set_cell_text(self, colx: Any, value: Any, style: Any = ...) -> None: ...
    def set_cell_blank(self, colx: Any, style: Any = ...) -> None: ...
    def set_cell_mulblanks(self, first_colx: Any, last_colx: Any, style: Any = ...) -> None: ...
    def set_cell_number(self, colx: Any, number: Any, style: Any = ...) -> None: ...
    def set_cell_date(self, colx: Any, datetime_obj: Any, style: Any = ...) -> None: ...
    def set_cell_formula(self, colx: Any, formula: Any, style: Any = ..., calc_flags: int = ...) -> None: ...
    def set_cell_boolean(self, colx: Any, value: Any, style: Any = ...) -> None: ...
    def set_cell_error(self, colx: Any, error_string_or_code: Any, style: Any = ...) -> None: ...
    def write(self, col: Any, label: Any, style: Any = ...) -> None: ...
    def set_cell_rich_text(self, col: Any, rich_text_list: Any, style: Any = ...) -> None: ...
    write_blanks: Any = ...
    write_rich_text: Any = ...



# Stubs for xlwt.Formatting (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class Font:
    ESCAPEMENT_NONE: int = ...
    ESCAPEMENT_SUPERSCRIPT: int = ...
    ESCAPEMENT_SUBSCRIPT: int = ...
    UNDERLINE_NONE: int = ...
    UNDERLINE_SINGLE: int = ...
    UNDERLINE_SINGLE_ACC: int = ...
    UNDERLINE_DOUBLE: int = ...
    UNDERLINE_DOUBLE_ACC: int = ...
    FAMILY_NONE: int = ...
    FAMILY_ROMAN: int = ...
    FAMILY_SWISS: int = ...
    FAMILY_MODERN: int = ...
    FAMILY_SCRIPT: int = ...
    FAMILY_DECORATIVE: int = ...
    CHARSET_ANSI_LATIN: int = ...
    CHARSET_SYS_DEFAULT: int = ...
    CHARSET_SYMBOL: int = ...
    CHARSET_APPLE_ROMAN: int = ...
    CHARSET_ANSI_JAP_SHIFT_JIS: int = ...
    CHARSET_ANSI_KOR_HANGUL: int = ...
    CHARSET_ANSI_KOR_JOHAB: int = ...
    CHARSET_ANSI_CHINESE_GBK: int = ...
    CHARSET_ANSI_CHINESE_BIG5: int = ...
    CHARSET_ANSI_GREEK: int = ...
    CHARSET_ANSI_TURKISH: int = ...
    CHARSET_ANSI_VIETNAMESE: int = ...
    CHARSET_ANSI_HEBREW: int = ...
    CHARSET_ANSI_ARABIC: int = ...
    CHARSET_ANSI_BALTIC: int = ...
    CHARSET_ANSI_CYRILLIC: int = ...
    CHARSET_ANSI_THAI: int = ...
    CHARSET_ANSI_LATIN_II: int = ...
    CHARSET_OEM_LATIN_I: int = ...
    height: int = ...
    italic: bool = ...
    struck_out: bool = ...
    outline: bool = ...
    shadow: bool = ...
    colour_index: int = ...
    bold: bool = ...
    escapement: Any = ...
    underline: Any = ...
    family: Any = ...
    charset: Any = ...
    name: str = ...
    def __init__(self) -> None: ...
    def get_biff_record(self): ...

class Alignment:
    HORZ_GENERAL: int = ...
    HORZ_LEFT: int = ...
    HORZ_CENTER: int = ...
    HORZ_RIGHT: int = ...
    HORZ_FILLED: int = ...
    HORZ_JUSTIFIED: int = ...
    HORZ_CENTER_ACROSS_SEL: int = ...
    HORZ_DISTRIBUTED: int = ...
    VERT_TOP: int = ...
    VERT_CENTER: int = ...
    VERT_BOTTOM: int = ...
    VERT_JUSTIFIED: int = ...
    VERT_DISTRIBUTED: int = ...
    DIRECTION_GENERAL: int = ...
    DIRECTION_LR: int = ...
    DIRECTION_RL: int = ...
    ORIENTATION_NOT_ROTATED: int = ...
    ORIENTATION_STACKED: int = ...
    ORIENTATION_90_CC: int = ...
    ORIENTATION_90_CW: int = ...
    ROTATION_0_ANGLE: int = ...
    ROTATION_STACKED: int = ...
    WRAP_AT_RIGHT: int = ...
    NOT_WRAP_AT_RIGHT: int = ...
    SHRINK_TO_FIT: int = ...
    NOT_SHRINK_TO_FIT: int = ...
    horz: Any = ...
    vert: Any = ...
    dire: Any = ...
    orie: Any = ...
    rota: Any = ...
    wrap: Any = ...
    shri: Any = ...
    inde: int = ...
    merg: int = ...
    def __init__(self) -> None: ...

class Borders:
    NO_LINE: int = ...
    THIN: int = ...
    MEDIUM: int = ...
    DASHED: int = ...
    DOTTED: int = ...
    THICK: int = ...
    DOUBLE: int = ...
    HAIR: int = ...
    MEDIUM_DASHED: int = ...
    THIN_DASH_DOTTED: int = ...
    MEDIUM_DASH_DOTTED: int = ...
    THIN_DASH_DOT_DOTTED: int = ...
    MEDIUM_DASH_DOT_DOTTED: int = ...
    SLANTED_MEDIUM_DASH_DOTTED: int = ...
    NEED_DIAG1: int = ...
    NEED_DIAG2: int = ...
    NO_NEED_DIAG1: int = ...
    NO_NEED_DIAG2: int = ...
    left: Any = ...
    right: Any = ...
    top: Any = ...
    bottom: Any = ...
    diag: Any = ...
    left_colour: int = ...
    right_colour: int = ...
    top_colour: int = ...
    bottom_colour: int = ...
    diag_colour: int = ...
    need_diag1: Any = ...
    need_diag2: Any = ...
    def __init__(self) -> None: ...

class Pattern:
    NO_PATTERN: int = ...
    SOLID_PATTERN: int = ...
    pattern: Any = ...
    pattern_fore_colour: int = ...
    pattern_back_colour: int = ...
    def __init__(self) -> None: ...

class Protection:
    cell_locked: int = ...
    formula_hidden: int = ...
    def __init__(self) -> None: ...



#!/usr/bin/env python3

import canmatrix.formats
from canmatrix.join import join_frame_by_signal_start_bit

files = ["../test/db_B.dbc", "../test/db_A.dbc"]

target = join_frame_by_signal_start_bit(files)

#
# export the new (target)-Matrix for example as .dbc:
#
canmatrix.formats.dumpp(target, "target.dbc")
canmatrix.formats.dumpp(target, "target.xlsx")



# Stubs for xlwt.compat (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

PY3: Any
unicode: Any
unicode_type = str
basestring = str
xrange = range
int_types: Any
long = int




#!/usr/bin/env python
# coding=utf-8
import codecs
import sys
sys.path.append('..')

import canmatrix
#
# create target Matrix
#

db = canmatrix.CanMatrix()

db.ecus.add(canmatrix.Ecu("testBU"))
db.ecus.add(canmatrix.Ecu("recBU"))

myFrame1 = canmatrix.Frame("canFdStandard1",Id=1,  dlc=24, is_fd = True, transmitter=["testBU"])
myFrame2 = canmatrix.Frame("CanFdExtended2",Id=2,  dlc=16, extended = True, is_fd = True, transmitter=["testBU"])
myFrame3 = canmatrix.Frame("CanExtended3", Id=3, dlc=8, extended = True, transmitter=["testBU"])
myFrame4 = canmatrix.Frame("CanStandard4", Id=4,  dlc=8)

mySignal1 = canmatrix.Signal("signal1", signalSize=64, startBit=0, is_little_endian=True, min=0, max=0, is_signed=True, receiver=["recBU"])
mySignal2 = canmatrix.Signal("signal2", signalSize=64, startBit=64, is_little_endian=True, min=0, max=0, is_signed=True, receiver=["recBU"])
mySignal3 = canmatrix.Signal("signal3", signalSize=64, startBit=128, is_little_endian=True, min=0, max=0, is_signed=True)

myFrame1.add_signal(mySignal3)
myFrame1.add_signal(mySignal2)
myFrame1.add_signal(mySignal1)


myFrame2.add_signal(mySignal2)
myFrame2.add_signal(mySignal1)


db.frames.add_frame(myFrame1)
db.frames.add_frame(myFrame2)
db.frames.add_frame(myFrame3)
db.frames.add_frame(myFrame4)


#
#
# export the new (target)-Matrix for example as .dbc:
#

canmatrix.formats.dumpp({"": db}, "testfd.dbc", dbcExportEncoding='iso-8859-1',
             dbcExportCommentEncoding='iso-8859-1')



#!/usr/bin/env python3

import canmatrix.formats
import sys
import optparse

# command line options...
usage = """
%prog [options]  matrix

matrixX can be any of *.dbc|*.dbf|*.kcd|*.arxml
"""

parser = optparse.OptionParser(usage=usage)
parser.add_option(
    "-f", "--frames",
    dest="frames",
    help="encode list of frames",
    default="*")

(cmdlineOptions, args) = parser.parse_args()

if len(args) < 1:
    parser.print_help()
    sys.exit(1)

# load matrix
db = canmatrix.formats.loadp_flat(args[0])

#get all frames which match the commandline
frames = db.glob_frames(cmdlineOptions.frames)

#helper to read physical value from user
def read_signal_value_from_user(signal):
    a = input("Enter Value for " + signal.name + " ")

    if signal.is_float:
        return float(a)
    else:
        return int(a)

# go through all frames
for frame in frames:
    print (frame.name)

    if frame.is_complex_multiplexed:
        # ignore complex multiplexed signals
        continue
    if frame.is_multiplexed:
        # if multiplexed frame search for multiplexer
        multiplexer_signal = frame.get_multiplexer

        # read multiplexer value
        a = input("Enter Value for Multiplexer " + multiplexer_signal.name + " ")
        signalDict = dict()
        signalDict[multiplexer_signal.name] = int(a)

        # read signals for the given multiplexer value
        for signal in frame.get_signals_for_multiplexer_value(int(a)):
            signalDict[signal.name] = read_signal_value_from_user(signal)

    else:
        # not multiplexed frame
        signalDict = dict()
        # go through all signals
        for signal in frame.signals:
            signalDict[signal.name] = read_signal_value_from_user(signal)

    frame_data = frame.encode(signalDict)
    if frame.arbitration_id.extended:
        print("{:05X}#".format(frame.arbitration_id.id) + "".join(["%02X" % i for i in frame_data]))
    else:
        print("{:03X}#".format(frame.arbitration_id.id) + "".join(["%02X" % i for i in frame_data]))



# Stubs for xlrd.book (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import struct
from typing import Any, Optional
from .biffh import BaseObject

unpack = struct.unpack
empty_cell: Any
DEBUG: int
USE_FANCY_CD: int
TOGGLE_GC: int
MMAP_AVAILABLE: int
USE_MMAP = MMAP_AVAILABLE
MY_EOF: int
SUPBOOK_UNK: Any
SUPBOOK_INTERNAL: Any
SUPBOOK_EXTERNAL: Any
SUPBOOK_ADDIN: Any
SUPBOOK_DDEOLE: Any
SUPPORTED_VERSIONS: Any
builtin_name_from_code: Any
code_from_builtin_name: Any

def open_workbook_xls(filename: Optional[Any] = ..., logfile: Any = ..., verbosity: int = ..., use_mmap: Any = ..., file_contents: Optional[Any] = ..., encoding_override: Optional[Any] = ..., formatting_info: bool = ..., on_demand: bool = ..., ragged_rows: bool = ...): ...

class Name(BaseObject):
    book: Any = ...
    hidden: int = ...
    func: int = ...
    vbasic: int = ...
    macro: int = ...
    complex: int = ...
    builtin: int = ...
    funcgroup: int = ...
    binary: int = ...
    name_index: int = ...
    name: Any = ...
    raw_formula: bytes = ...
    scope: int = ...
    result: Any = ...
    def cell(self): ...
    def area2d(self, clipped: bool = ...): ...

class Book(BaseObject):
    nsheets: int = ...
    datemode: int = ...
    biff_version: int = ...
    name_obj_list: Any = ...
    codepage: Any = ...
    encoding: Any = ...
    countries: Any = ...
    user_name: Any = ...
    font_list: Any = ...
    xf_list: Any = ...
    format_list: Any = ...
    format_map: Any = ...
    style_name_map: Any = ...
    colour_map: Any = ...
    palette_record: Any = ...
    load_time_stage_1: Any = ...
    load_time_stage_2: Any = ...
    def sheets(self): ...
    def sheet_by_index(self, sheetx: Any): ...
    def sheet_by_name(self, sheet_name: Any): ...
    def sheet_names(self): ...
    def sheet_loaded(self, sheet_name_or_index: Any): ...
    def unload_sheet(self, sheet_name_or_index: Any) -> None: ...
    mem: Any = ...
    filestr: Any = ...
    def release_resources(self) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type: Any, exc_value: Any, exc_tb: Any) -> None: ...
    name_and_scope_map: Any = ...
    name_map: Any = ...
    raw_user_name: bool = ...
    builtinfmtcount: int = ...
    addin_func_names: Any = ...
    def __init__(self) -> None: ...
    logfile: Any = ...
    verbosity: Any = ...
    use_mmap: Any = ...
    encoding_override: Any = ...
    formatting_info: Any = ...
    on_demand: Any = ...
    ragged_rows: Any = ...
    stream_len: Any = ...
    base: int = ...
    def biff2_8_load(self, filename: Optional[Any] = ..., file_contents: Optional[Any] = ..., logfile: Any = ..., verbosity: int = ..., use_mmap: Any = ..., encoding_override: Optional[Any] = ..., formatting_info: bool = ..., on_demand: bool = ..., ragged_rows: bool = ...) -> None: ...
    xfcount: int = ...
    actualfmtcount: int = ...
    def initialise_format_info(self) -> None: ...
    def get2bytes(self): ...
    def get_record_parts(self): ...
    def get_record_parts_conditional(self, reqd_record: Any): ...
    def get_sheet(self, sh_number: Any, update_pos: bool = ...): ...
    def get_sheets(self) -> None: ...
    def fake_globals_get_sheet(self) -> None: ...
    def handle_boundsheet(self, data: Any) -> None: ...
    def handle_builtinfmtcount(self, data: Any) -> None: ...
    def derive_encoding(self): ...
    def handle_codepage(self, data: Any) -> None: ...
    def handle_country(self, data: Any) -> None: ...
    def handle_datemode(self, data: Any) -> None: ...
    def handle_externname(self, data: Any) -> None: ...
    def handle_externsheet(self, data: Any) -> None: ...
    def handle_filepass(self, data: Any) -> None: ...
    def handle_name(self, data: Any) -> None: ...
    def names_epilogue(self) -> None: ...
    def handle_obj(self, data: Any) -> None: ...
    def handle_supbook(self, data: Any) -> None: ...
    def handle_sheethdr(self, data: Any) -> None: ...
    def handle_sheetsoffset(self, data: Any) -> None: ...
    def handle_sst(self, data: Any) -> None: ...
    def handle_writeaccess(self, data: Any) -> None: ...
    def parse_globals(self) -> None: ...
    def read(self, pos: Any, length: Any): ...
    def getbof(self, rqd_stream: Any): ...

def expand_cell_address(inrow: Any, incol: Any): ...
def colname(colx: Any, _A2Z: str = ...): ...
def display_cell_address(rowx: Any, colx: Any, relrow: Any, relcol: Any): ...
def unpack_SST_table(datatab: Any, nstrings: Any): ...



#!/usr/bin/env python
# Copyright (c) 2016, Eduard Broecker
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that
# the following conditions are met:
#
#    Redistributions of source code must retain the above copyright notice, this list of conditions and the
#    following disclaimer.
#    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
#    following disclaimer in the documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.

import sys
sys.path.append('..')
import canmatrix.formats
from createCMacros import *


typedef = """
typedef struct Mailbox {
uint8_t data[8];
} Mailbox_t;
"""

processFrame = """
void processFrame(uint32_t canId, uint8_t *framedata, uint8_t dlc)
{
    Mailbox_t *mailbox;

    mailbox = getRxCanMailBox(canId);

    if (mailbox != NULL)
    {
        memcpy(mailbox->data, framedata, (dlc>8)?8:dlc);
    }
}
"""

receiveIdArray = "uint32_t receiveFrameIds[COUNTRXMAILBOXES] = {%s};"
rxMailboxStruct = "Mailbox_t RxCanMailboxes[COUNTRXMAILBOXES];"
txMailboxStruct = "Mailbox_t TxCanMailboxes[COUNTTXMAILBOXES];"

getRxCanMailBox = """
struct Mailbox *getRxCanMailBox(uint32_t canId)
{
    uint32_t i;

    for(i = 0; i< COUNTRXMAILBOXES; i++)
    {
        if(canId == receiveFrameIds[i])
            return &(RxCanMailboxes[i]);
    }
    return 0;
}

"""


def main():
    if len(sys.argv) <= 2:
        print "not yet working script for generating a communication layer for dedicated ECU out of can database"
        print "! missing support for sending cyclic messages"
        print "! missing any TEST ! "
        print " this code is just proofe of concept \n"
        print "Usage: createccl.py CanDatabaseFile ECU_Name "
        exit()

    ccl_h = "#ifndef __CCL_H__\n#define __CCL_H__\n\n"
    ccl_h += "typedef unsigned char uint8;\ntypedef unsigned int uint32;\n\n"

    ccl_c = "#include <string.h>\n#include <stdint.h>\n#include \"ccl.h\"\n\n"
    infile = sys.argv[1]
    ecu = sys.argv[2]

    dbs = canmatrix.formats.loadp(infile)
    db = next(iter(dbs.values()))

    receiveArray = []
    receiveDict = {}
    receiveIndex = 0
    sendIndex = 0
    txDict = {}
    for frame in db.frames:
        if ecu in frame.receivers:
            receiveArray.append(frame.arbitration_id.id)
            receiveDict[frame] = receiveIndex
            receiveIndex += 1

        if ecu in frame.transmitters:
            txDict[frame] = sendIndex
            sendIndex += 1
#            print frame.name
#            if  frame.effective_cycletime != 0:
#                print frame.name,
#                print frame.effective_cycletime
#            ccl_h += createStoreMacrosForFrame(frame, "_" + frame.name + "_")

    tempStr = ""
    for canid in receiveArray:
        tempStr += hex(canid) + ", "

    ccl_c += rxMailboxStruct + "\n"
    ccl_c += txMailboxStruct + "\n"
    ccl_c += receiveIdArray % (tempStr) + "\n"
    ccl_c += getRxCanMailBox + "\n"
    ccl_c += processFrame + "\n"

    ccl_h += "#define COUNTRXMAILBOXES %d" % (receiveArray.__len__()) + "\n"
    ccl_h += "#define COUNTTXMAILBOXES %d" % (txDict.__len__()) + "\n"
    ccl_h += typedef + "\n"
    ccl_h += "extern " + rxMailboxStruct + "\n"
    ccl_h += "extern " + txMailboxStruct + "\n"
    for frame in receiveDict:
        for signal in frame.signals:
            if ecu in signal.receivers:
                ccl_h += createDecodeMacro(signal, "_" +
                                           frame.name +
                                           "__", "", "RxCanMailboxes[%d].data" %
                                           (receiveDict[frame]))
# for complete frame:
#        ccl_h += createDecodeMacrosForFrame(frame, "_" + frame.name + "__", "", "RxCanMailboxes[%d].data" % (receiveDict[frame]))
    for frame in txDict:
        ccl_h += createStoreMacrosForFrame(frame, "_" +
                                           frame.name +
                                           "_", framename="TxCanMailboxes[%d].data" %
                                           txDict[frame])

    ccl_h += "\n\n#endif /*  __CCL_H___ */"

    cfile = open("ccl.c", "w")
    cfile.write(ccl_c)
    cfile.close()

    hfile = open("ccl.h", "w")
    hfile.write(ccl_h)
    hfile.close()

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3
import canmatrix.formats
import sys
cluster = canmatrix.formats.loadp(sys.argv[1], decode_flexray = True)

for cm in cluster:
    for frame in cluster[cm]:
        frame_info = "{}-{}-{}".format(frame.slot_id, frame.base_cycle, frame.repitition_cycle)
        for pdu in frame.pdus:
            for signal in pdu.signals:
                sig_group = pdu.get_signal_group_for_signal(signal)
                sig_group = "None" if sig_group is None else sig_group.name
                print("{}: {}, {}, {}, {}, {}".format(frame_info, frame.size, pdu.pdu_type, pdu.name, sig_group, signal.name))




# Stubs for xlrd.xlsx (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .timemachine import *
from typing import Any, Optional

DEBUG: int
DLF: Any
ET: Any
ET_has_iterparse: bool
Element_has_iter: bool

def ensure_elementtree_imported(verbosity: Any, logfile: Any) -> None: ...
def split_tag(tag: Any): ...
def augment_keys(adict: Any, uri: Any) -> None: ...
def cell_name_to_rowx_colx(cell_name: Any, letter_value: Any = ..., allow_no_col: bool = ...): ...

error_code_from_text: Any
U_SSML12: str
U_ODREL: str
U_PKGREL: str
U_CP: str
U_DC: str
U_DCTERMS: str
XML_SPACE_ATTR: str
XML_WHITESPACE: str
X12_MAX_ROWS: Any
X12_MAX_COLS: Any
V_TAG: Any
F_TAG: Any
IS_TAG: Any

def unescape(s: Any, subber: Any = ..., repl: Any = ...): ...
def cooked_text(self, elem: Any): ...
def get_text_from_si_or_is(self, elem: Any, r_tag: Any = ..., t_tag: Any = ...): ...
def map_attributes(amap: Any, elem: Any, obj: Any) -> None: ...
def cnv_ST_Xstring(s: Any): ...
def cnv_xsd_unsignedInt(s: Any): ...
def cnv_xsd_boolean(s: Any): ...
def make_name_access_maps(bk: Any) -> None: ...

class X12General:
    tree: Any = ...
    def process_stream(self, stream: Any, heading: Optional[Any] = ...) -> None: ...
    def finish_off(self) -> None: ...
    def dump_elem(self, elem: Any) -> None: ...
    def dumpout(self, fmt: Any, *vargs: Any) -> None: ...

class X12Book(X12General):
    bk: Any = ...
    logfile: Any = ...
    verbosity: Any = ...
    relid2path: Any = ...
    relid2reltype: Any = ...
    sheet_targets: Any = ...
    sheetIds: Any = ...
    def __init__(self, bk: Any, logfile: Any = ..., verbosity: bool = ...) -> None: ...
    core_props_menu: Any = ...
    tree: Any = ...
    def process_coreprops(self, stream: Any) -> None: ...
    @staticmethod
    def convert_filename(name: Any): ...
    def process_rels(self, stream: Any) -> None: ...
    def do_defined_name(self, elem: Any) -> None: ...
    def do_defined_names(self, elem: Any) -> None: ...
    def do_sheet(self, elem: Any) -> None: ...
    def do_workbookpr(self, elem: Any) -> None: ...
    tag2meth: Any = ...

class X12SST(X12General):
    bk: Any = ...
    logfile: Any = ...
    verbosity: Any = ...
    process_stream: Any = ...
    def __init__(self, bk: Any, logfile: Any = ..., verbosity: int = ...) -> None: ...
    def process_stream_iterparse(self, stream: Any, heading: Optional[Any] = ...) -> None: ...
    tree: Any = ...
    def process_stream_findall(self, stream: Any, heading: Optional[Any] = ...) -> None: ...

class X12Styles(X12General):
    bk: Any = ...
    logfile: Any = ...
    verbosity: Any = ...
    xf_counts: Any = ...
    xf_type: Any = ...
    fmt_is_date: Any = ...
    def __init__(self, bk: Any, logfile: Any = ..., verbosity: int = ...) -> None: ...
    def do_cellstylexfs(self, elem: Any) -> None: ...
    def do_cellxfs(self, elem: Any) -> None: ...
    def do_numfmt(self, elem: Any) -> None: ...
    def do_xf(self, elem: Any) -> None: ...
    tag2meth: Any = ...

class X12Sheet(X12General):
    sheet: Any = ...
    logfile: Any = ...
    verbosity: Any = ...
    rowx: int = ...
    bk: Any = ...
    sst: Any = ...
    relid2path: Any = ...
    relid2reltype: Any = ...
    merged_cells: Any = ...
    warned_no_cell_name: int = ...
    warned_no_row_num: int = ...
    process_stream: Any = ...
    def __init__(self, sheet: Any, logfile: Any = ..., verbosity: int = ...) -> None: ...
    def own_process_stream(self, stream: Any, heading: Optional[Any] = ...) -> None: ...
    def process_rels(self, stream: Any) -> None: ...
    def process_comments_stream(self, stream: Any) -> None: ...
    def do_dimension(self, elem: Any) -> None: ...
    def do_merge_cell(self, elem: Any) -> None: ...
    def do_row(self, row_elem: Any) -> None: ...
    tag2meth: Any = ...

def open_workbook_2007_xml(zf: Any, component_names: Any, logfile: Any = ..., verbosity: int = ..., use_mmap: int = ..., formatting_info: int = ..., on_demand: int = ..., ragged_rows: int = ...): ...



# Hand-written stub for lxml.etree as used by mypy.report.
# This is *far* from complete, and the stubgen-generated ones crash mypy.
# Any use of `Any` below means I couldn't figure out the type.

import typing
from typing import Any, Dict, List, Mapping, MutableMapping, Tuple, Union, Optional, Sequence
from typing import Iterable, Iterator, SupportsBytes


# We do *not* want `typing.AnyStr` because it is a `TypeVar`, which is an
# unnecessary constraint. It seems reasonable to constrain each
# List/Dict argument to use one type consistently, though, and it is
# necessary in order to keep these brief.
_AnyStr = Union[str]
_ListAnyStr = Union[List[str], List[bytes]]
_DictAnyStr = Union[Dict[str, str], Dict[bytes, bytes]]
_Dict_Tuple2AnyStr_Any = Union[Dict[Tuple[str, str], Any], Tuple[bytes, bytes], Any]
_xpath = Union['XPath', _AnyStr]
_OptionalNamespace = Optional[Mapping[str, Any]]

class ElementChildIterator(Iterator['_Element']):
    def __iter__(self) -> 'ElementChildIterator': ...
    def __next__(self) -> '_Element': ...

class _Element(Iterable['_Element']):
    def addprevious(self, element: '_Element') -> None: ...
    def addnext(self, element: '_Element') -> None: ...
    def find(self, path: str, namespace: _OptionalNamespace = None)  -> '_Element': ...
    def findall(self, name: str, namespace: _OptionalNamespace = None) -> Sequence['_Element']: ...
    def clear(self) -> None: ...
    def get(self, key: _AnyStr, default: Optional[_AnyStr] = ...) -> _AnyStr: ...
    def set(self, key: _AnyStr, value: _AnyStr) -> None: ...
    def xpath(self, _path: _AnyStr, namespaces: Optional[_DictAnyStr] = ..., extensions: Any = ..., smart_strings: bool = ..., **_variables: Any) -> Any: ...
    # indeed returns a Union[bool, float, _AnyStr, List[Union[ElementBase, _AnyStr, Tuple[]]]]: ...
    # http://lxml.de/xpathxslt.html#xpath-return-values
    attrib = ...  # type: MutableMapping[str, str]
    text = ...  # type: _AnyStr
    tag = ...  # type: str
    sourceline = ...  # type: Optional[int]
    def append(self, element: '_Element') -> '_Element': ...
    def __iter__(self) -> ElementChildIterator: ...
    def items(self) -> Sequence[Tuple[_AnyStr, _AnyStr]]: ...
    def iterfind(self, path: str, namespace: _OptionalNamespace = None)  -> Iterator['_Element']: ...
    def getparent(self) -> _Element: ...
    def __len__(self) -> int: ...

class ElementBase(_Element): ...

class _ElementTree:
    def write(self,
              file: Union[_AnyStr, typing.IO],
              encoding: _AnyStr = ...,
              method: _AnyStr = ...,
              pretty_print: bool = ...,
              xml_declaration: Any = ...,
              with_tail: Any = ...,
              standalone: bool = ...,
              compression: int = ...,
              exclusive: bool = ...,
              with_comments: bool = ...,
              inclusive_ns_prefixes: _ListAnyStr = ...) -> None: ...
    def xpath(self, _path: _AnyStr, namespaces: Optional[_DictAnyStr] = ..., extensions: Any = ..., smart_strings: bool = ..., **_variables: Any) -> Any: ...
    def getroot(self) -> _Element: ...

class _XSLTResultTree(SupportsBytes):
    def __bytes__(self) -> bytes: ...

class _XSLTQuotedStringParam: ...

class XMLParser: ...

class XMLSchema:
    def __init__(self,
                 etree: Union[_Element, _ElementTree] = ...,
                 file: Union[_AnyStr, typing.IO] = ...) -> None: ...
    def assertValid(self, etree: Union[_Element, _ElementTree]) -> None: ...

class XSLTAccessControl: ...

class XSLT:
    def __init__(self,
                 xslt_input: Union[_Element, _ElementTree],
                 extensions: _Dict_Tuple2AnyStr_Any = ...,
                 regexp: bool = ...,
                 access_control: XSLTAccessControl = ...) -> None: ...
    def __call__(self,
                 _input: Union[_Element, _ElementTree],
                 profile_run: bool = ...,
                 **kwargs: Union[_AnyStr, _XSLTQuotedStringParam]) -> _XSLTResultTree: ...
    @staticmethod
    def strparam(s: _AnyStr) -> _XSLTQuotedStringParam: ...

def Element(_tag: _AnyStr,
            attrib: _DictAnyStr = ...,
            nsmap: _DictAnyStr = ...,
            **extra: _AnyStr) -> _Element: ...
def SubElement(_parent: _Element, _tag: _AnyStr,
               attrib: _DictAnyStr = ...,
               nsmap: _DictAnyStr = ...,
               **extra: _AnyStr) -> _Element: ...
def ElementTree(element: _Element = ...,
                file: Union[_AnyStr, typing.IO] = ...,
                parser: XMLParser = ...) -> _ElementTree: ...
def ProcessingInstruction(target: _AnyStr, text: _AnyStr = ...) -> _Element: ...
def parse(source: Union[_AnyStr, typing.IO],
          parser: XMLParser = ...,
          base_url: _AnyStr = ...) -> _ElementTree: ...
def fromstring(text: _AnyStr,
               parser: XMLParser = ...,
               *,
               base_url: _AnyStr = ...) -> _Element: ...
def tostring(element_or_tree: Union[_Element, _ElementTree],
             encoding: Union[str, type] = ...,
             method: str = ...,
             xml_declaration: bool = ...,
             pretty_print: bool = ...,
             with_tail: bool = ...,
             standalone: bool = ...,
             doctype: str = ...,
             exclusive: bool = ...,
             with_comments: bool = ...,
             inclusive_ns_prefixes: Any = ...) -> _AnyStr: ...

class _ErrorLog: ...

class Error(Exception): ...

class LxmlError(Error):
    def __init__(self, message: Any, error_log: _ErrorLog = ...) -> None: ...
    error_log = ...  # type: _ErrorLog

class DocumentInvalid(LxmlError): ...
class LxmlSyntaxError(LxmlError, SyntaxError): ...
class ParseError(LxmlSyntaxError): ...
class XMLSyntaxError(ParseError): ...

class _Validator: ...

class DTD(_Validator):
    def __init__(self,
                 file: Union[_AnyStr, typing.IO] = ...,
                 *,
                 external_id: Any = ...) -> None: ...

    def assertValid(self, etree: _Element) -> None: ...

class XPath:
    def __init__(self, path: _AnyStr, namespaces: Optional[_AnyStr], extensions: Optional[_AnyStr], regexp: Optional[bool], smart_strings: Optional[bool]) -> None: ...



# Stubs for xlrd (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .biffh import XLRDError, XL_CELL_BLANK, XL_CELL_BOOLEAN, XL_CELL_DATE, XL_CELL_EMPTY, XL_CELL_ERROR, XL_CELL_NUMBER, XL_CELL_TEXT, biff_text_from_num, error_text_from_code
from .book import Book as Book, colname as colname
from .sheet import empty_cell as empty_cell
from .xldate import XLDateError, xldate_as_datetime, xldate_as_tuple
from .xlsx import X12Book as X12Book
from typing import Any, Optional

MMAP_AVAILABLE: int
USE_MMAP = MMAP_AVAILABLE

def open_workbook(filename: Optional[Any] = ..., logfile: Any = ..., verbosity: int = ..., use_mmap: Any = ..., file_contents: Optional[Any] = ..., encoding_override: Optional[Any] = ..., formatting_info: bool = ..., on_demand: bool = ..., ragged_rows: bool = ...): ...
def dump(filename: Any, outfile: Any = ..., unnumbered: bool = ...) -> None: ...
def count_records(filename: Any, outfile: Any = ...) -> None: ...



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `canmatrix` package."""
import unittest
import tempfile
import os

from canmatrix import formats
from canmatrix.canmatrix import Signal


class TestCanmatrixCodec(unittest.TestCase):
    """Tests for `canmatrix` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_bitstruct_format(self):
        """"""
        s1 = Signal('signal')
        self.assertEqual(s1.bitstruct_format(), '<s0')

        s2 = Signal('signal', size=8)
        self.assertEqual(s2.bitstruct_format(), '<s8')

        s3 = Signal('signal', size=8, is_signed=False)
        self.assertEqual(s3.bitstruct_format(), '<u8')

        s4 = Signal('signal', size=8, is_little_endian=False)
        self.assertEqual(s4.bitstruct_format(), '>s8')

    def test_encode_canmatrix(self):
        db_path = os.path.join(
            os.path.dirname(__file__), "..", "test", "test.dbc")
        for bus in formats.loadp(db_path).values():
            test_frame1 = 0x123
            data = {
                'Signal': 2,
                'someTestSignal': 101,
            }
            data_bytes = tuple(bytearray(bus.encode(test_frame1, data)))
            assert data_bytes == (0, 0x28, 0x04, 0, 0, 0, 0, 0)

    def test_encode_decode_signal_value(self):
        db_path = os.path.join(
            os.path.dirname(__file__), "..", "test", "test.dbc")
        for bus in formats.loadp(db_path).values():
            test_frame1 = 0x123

            data = {
                'Signal': 2,
                'someTestSignal': 101,
            }
            data_bytes = tuple(bytearray(bus.encode(test_frame1, data)))
            decoded = bus.decode(test_frame1, data_bytes, False)

            for k, v in data.items():
                assert decoded[k] == v

    def test_encode_decode_signal_value_choice_unicode(self):
        db_path = os.path.join(
            os.path.dirname(__file__), "..", "test", "test.dbc")
        for bus in formats.loadp(db_path).values():
            test_frame1 = 0x123

            data = {
                'Signal': u'two'
            }
            data_bytes = tuple(bytearray(bus.encode(test_frame1, data)))

            decoded = bus.decode(test_frame1, data_bytes, True)

            for k, v in data.items():
                assert str(decoded[k]) == v

    def test_encode_decode_signal_value_choice_str(self):
        db_path = os.path.join(
            os.path.dirname(__file__), "..", "test", "test.dbc")
        for bus in formats.loadp(db_path).values():
            test_frame1 = 0x123

            data = {
                'Signal': 'two'
            }
            data_bytes = tuple(bytearray(bus.encode(test_frame1, data)))

            decoded = bus.decode(test_frame1, data_bytes, True)

            for k, v in data.items():
                assert str(decoded[k]) == v

    def test_import_export_additional_frame_info(self):
        db_path = os.path.join(
            os.path.dirname(__file__), "..", "test", "test.dbc")
        dbs = formats.loadp(db_path)
        tmp_dir = tempfile.mkdtemp()
        for extension in ['csv', 'json']:
            out_file_name = tmp_dir + "/output." + extension
            formats.dumpp(dbs, out_file_name, additionalFrameAttributes="UserFrameAttr")
            with open(out_file_name, "r") as file:
                data = file.read()
            self.assertIn("UserFrameAttr", data)


if __name__ == "__main__":
    unittest.main()



# Stubs for xlwt.Workbook (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .Worksheet import Worksheet

class Workbook:
    encoding: Any = ...
    def __init__(self, encoding: str = ..., style_compression: int = ...) -> None: ...
    def get_style_stats(self): ...
    def set_owner(self, value: Any) -> None: ...
    def get_owner(self): ...
    owner: Any = ...
    def set_country_code(self, value: Any) -> None: ...
    def get_country_code(self): ...
    country_code: Any = ...
    def set_wnd_protect(self, value: Any) -> None: ...
    def get_wnd_protect(self): ...
    wnd_protect: Any = ...
    def set_obj_protect(self, value: Any) -> None: ...
    def get_obj_protect(self): ...
    obj_protect: Any = ...
    def set_protect(self, value: Any) -> None: ...
    def get_protect(self): ...
    protect: Any = ...
    def set_backup_on_save(self, value: Any) -> None: ...
    def get_backup_on_save(self): ...
    backup_on_save: Any = ...
    def set_hpos(self, value: Any) -> None: ...
    def get_hpos(self): ...
    hpos: Any = ...
    def set_vpos(self, value: Any) -> None: ...
    def get_vpos(self): ...
    vpos: Any = ...
    def set_width(self, value: Any) -> None: ...
    def get_width(self): ...
    width: Any = ...
    def set_height(self, value: Any) -> None: ...
    def get_height(self): ...
    height: Any = ...
    def set_active_sheet(self, value: Any) -> None: ...
    def get_active_sheet(self): ...
    active_sheet: Any = ...
    def set_tab_width(self, value: Any) -> None: ...
    def get_tab_width(self): ...
    tab_width: Any = ...
    def set_wnd_visible(self, value: Any) -> None: ...
    def get_wnd_visible(self): ...
    wnd_visible: Any = ...
    def set_wnd_mini(self, value: Any) -> None: ...
    def get_wnd_mini(self): ...
    wnd_mini: Any = ...
    def set_hscroll_visible(self, value: Any) -> None: ...
    def get_hscroll_visible(self): ...
    hscroll_visible: Any = ...
    def set_vscroll_visible(self, value: Any) -> None: ...
    def get_vscroll_visible(self): ...
    vscroll_visible: Any = ...
    def set_tabs_visible(self, value: Any) -> None: ...
    def get_tabs_visible(self): ...
    tabs_visible: Any = ...
    def set_dates_1904(self, value: Any) -> None: ...
    def get_dates_1904(self): ...
    dates_1904: Any = ...
    def set_use_cell_values(self, value: Any) -> None: ...
    def get_use_cell_values(self): ...
    use_cell_values: Any = ...
    def get_default_style(self): ...
    default_style: Any = ...
    def set_colour_RGB(self, colour_index: Any, red: Any, green: Any, blue: Any) -> None: ...
    def add_style(self, style: Any): ...
    def add_font(self, font: Any): ...
    def add_str(self, s: Any): ...
    def del_str(self, sst_idx: Any) -> None: ...
    def str_index(self, s: Any): ...
    def add_rt(self, rt: Any): ...
    def rt_index(self, rt: Any): ...
    def add_sheet(self, sheetname: str, cell_overwrite_ok: bool = ...) -> Worksheet: ...
    def get_sheet(self, sheet: Any): ...
    def sheet_index(self, sheetname: Any): ...
    def raise_bad_sheetname(self, sheetname: Any) -> None: ...
    def convert_sheetindex(self, strg_ref: Any, n_sheets: Any): ...
    def setup_ownbook(self) -> None: ...
    def setup_xcall(self) -> None: ...
    def add_sheet_reference(self, formula: Any) -> None: ...
    def get_biff_data(self): ...
    def save(self, filename_or_stream: Any) -> None: ...



# Stubs for xlwt (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from .Column import Column as Column
from .Formatting import Alignment, Borders, Font, Pattern, Protection
from .Row import Row as Row
from .Style import XFStyle as XFStyle, easyfont as easyfont, easyxf as easyxf
from .Workbook import Workbook as Workbook
from .Worksheet import Worksheet as Worksheet


__VERSION__: str



# Stubs for xlwt.UnicodeUtils (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

def upack2(s: Any, encoding: str = ...): ...
def upack2rt(rt: Any, encoding: str = ...): ...
def upack1(s: Any, encoding: str = ...): ...
